#!/usr/bin/perl

#
# Author: papanda@papa.to
# License: 3-clause BSD license
#

use strict;
use CGI;
use DateTime;
use Unicode::Japanese;
use Time::HiRes qw( gettimeofday tv_interval );
use Data::Dumper;
use JSON;
use DBI;
use utf8;

require 'nibbleconf.pl';
require 'searchconf.pl';
require 'searchutil.pl';
require 'jsonutil.pl';
require 'wordutil.pl';

my $recdays = $searchconf::tsexpiredays;

my $now = time();
my $t0 = [gettimeofday];
my $op = {};
my $cachesec = $searchconf::cachesec;

my $q = new CGI;
my $remoteaddr = $q->remote_addr();

my $networkid = $q->param('networkid') || 0;
if (!$networkid) {
    print &jsonutil::output($q, 406, '{ "error": "invalid networkid" }');
    exit 0;
}
my $serviceid = $q->param('serviceid') || 0;
my $ptime = $q->param('ptime') || $now;
$ptime = ($now - 45) if ($ptime > ($now - 45));
my $pdt = DateTime->from_epoch( epoch=>$ptime, time_zone=>'local' );
my $serviceid = $q->param('serviceid') || 0;

my @result = ();
my $sdt = DateTime->now( time_zone=>'local' );
$sdt->subtract( days=>$recdays ); 

######################### INIT ######################################
my $dsn = 'DBI:SQLite:dbname='.$searchconf::dbpath;
my $dbh = DBI->connect($dsn, undef, undef, {RaiseError => 1, AutoCommit => 0});

$op->{seladdr} = '127.0.0.1';
$op->{etag} = 'sqlite3';
$op->{num} = 0;

my $nstable = {};
my $chtable = {};
my $sql = "select dc_ch,dc_name,dc_network_id,dc_service_id from da_channel;";
my $sth = $dbh->prepare($sql);
my $rv = $sth->execute();
while (my $rcols = $sth->fetch()) {
    my ($dc_ch, $dc_name, $dc_network_id, $dc_service_id) = @$rcols;
    $nstable->{$dc_network_id . ':' . $dc_service_id} = {
	'phych'=>$dc_ch,
	'provider'=>$dc_name,
    };
    $chtable->{$dc_ch} = {
	'networkid'=>$dc_network_id,
	'serviceid'=>$dc_service_id,
	'provider'=>$dc_name,
    };
}

if ($q->param('starttime') && $q->param('endtime')) {
    my $bsdt = &searchutil::convtimestr($q->param('starttime'));
    my $bedt = &searchutil::convtimestr($q->param('endtime'));
    if ($bsdt < $bedt) {
	$pdt = $bedt->clone();
	$pdt->subtract( minutes=>1 );
    }
}

my $starttime = &searchutil::sqlite3date($sdt);
my $pointtime = &searchutil::sqlite3date($pdt);

my $epg = {};
if (1) {
    my $sql = 'select p.dv_id,p.dv_service_id,p.dv_original_network_id,p.dv_event_id,p.dv_start_time,datetime(p.dv_start_time, p.dv_duration),p.dv_name,p.dv_text, (select g.mg_level1 from da_genre g where g.dv_id = p.dv_id order by g.dg_id limit 1) from da_event p where p.dv_start_time > ? and p.dv_start_time <= ? and datetime(p.dv_start_time, p.dv_duration) > ? and p.dv_original_network_id = ? order by p.dv_start_time limit 16';
    my $sth = $dbh->prepare($sql);
    my $rv = $sth->execute($starttime, $pointtime, $pointtime, $networkid);
    while (my $rcols = $sth->fetch()) {
	my ($dv_id, 
	    $dv_service_id,
	    $dv_original_network_id,
	    $dv_event_id,
	    $dv_start_time,
	    $dv_end_time,
	    $dv_name,
	    $dv_text,
	    $dv_nib) = @$rcols;
	my $stime1 = &searchutil::sqlite3rev($dv_start_time);
	my $etime1 = &searchutil::sqlite3rev($dv_end_time);
	next if (!exists $nstable->{$dv_original_network_id . ':' . $dv_service_id});
	my $phych = $nstable->{$dv_original_network_id . ':' . $dv_service_id}->{'phych'};
	my $provider = $nstable->{$dv_original_network_id . ':' . $dv_service_id}->{'provider'};
	utf8::decode($provider);
	my $epgid = $dv_original_network_id . ':' . $dv_service_id . ':' . $dv_event_id;
	my $vgenre = $nibbleconf::table{$dv_nib};
	utf8::decode($vgenre);
	my $vtitle = $dv_name;
	utf8::decode($vtitle);
	&wordutil::ctrlesc(\$vtitle);
	my $vdesc = $dv_text;
	utf8::decode($vdesc);
	&wordutil::ctrlesc(\$vdesc);
	my $oneseg24session = sprintf("%02d%02d%02d-%02d%02d%02d-%02d%02d%02d-%d", 
	    $stime1->year - 2000,
	    $stime1->month,
	    $stime1->day,
	    $stime1->hour,
	    $stime1->minute,
	    $stime1->second,
	    $etime1->hour,
	    $etime1->minute,
	    $etime1->second,
	    $phych);
	$epg->{$epgid} = {
	    'networkid'=>sprintf("%d", $dv_original_network_id),
	    'serviceid'=>sprintf("%d", $dv_service_id),
	    'eventid'=>sprintf("%d", $dv_event_id),
	    'title'=>$vtitle,
	    'desc'=>$vdesc,
	    'start'=>sprintf("%d", $stime1->epoch),
	    'end'=>sprintf("%d", $etime1->epoch),
	    'provider'=>$provider,
	    'genre'=>$vgenre,
	    'session'=>$oneseg24session,
	    'protect'=>'1',
	};
    }
}

########################### RESULT #############################
# 先に閉じておく
$dbh->disconnect();

if ($serviceid >= 16) {
    foreach my $epgid (sort { $epg->{$a}->{start} <=> $epg->{$b}->{start} } keys %$epg) {
	my $serviceid0 = $epg->{$epgid}->{serviceid};
	if ($serviceid0 == $serviceid) {
	    push(@result, $epg->{$epgid});
	}
    }
}
if (scalar @result == 0) {
    @result = %$epg;
}

$op->{num} = sprintf("%d", scalar @result);
$op->{result} = \@result;
############################## OUTPUT ######################################
# 出力
my $output = JSON->new->utf8(0)->encode($op);
utf8::encode($output); # UTF8フラグを外す
print &jsonutil::output($q, 200, $output, 'text/json', $cachesec);

1;
