#!/usr/bin/perl

#
# Author: papanda@papa.to
# License: 3-clause BSD license
#

use lib ('/mnt/oneseg24/webapi2', '/mnt/rtmpserver/lib');

use strict;
use CGI;
use DBI;
use RTMP;
use FileHandle;
use IO::Socket;
use DateTime;
use Unicode::Japanese;
use Time::HiRes qw( gettimeofday tv_interval );
use URI::Escape;
use Data::Dumper;

require 'searchconf.pl';

$tsdownload::packsize = 32768;
$tsdownload::tsdir = $ENV{'TSDIR'} || '/mnt/ts';
$tsdownload::contenttype = 'video/ts';

# ストリームの保存期間(日数)
my $recdays = $searchconf::tsexpiredays;
my $sdt0 = DateTime->now( time_zone=>'local' );
$sdt0->subtract( days=>$recdays ); 

my $io = new IO::Handle;
$io->fdopen(fileno(STDOUT), "w");
my $q = new CGI;

my $title = $q->param('title');
my $numb = $q->param('numb');
my $networkid = $q->param('networkid');
my $serviceid = $q->param('serviceid');
my $stime = $q->param('stime');
my $etime = $q->param('etime');
my $phych = $q->param('phych');
my $hdtv = $q->param('hdtv') || 0;

if ($stime >= $etime || !$networkid || !$serviceid) {
    print $q->header(-type=>$tsdownload::contenttype, -status=>406, -charset=>'utf-8', -connection=>'close', -cachecontrol=>'no-cache', -pragma=>'no-cache', -expires=>'-1');
    exit 1;
}

if ($phych < 13) {
    my $dsn = 'DBI:SQLite:dbname='.$searchconf::dbpath;
    my $dbh = DBI->connect($dsn, undef, undef, {RaiseError => 1, AutoCommit => 0});
    my $nstable = {};
    my $chtable = {};
    my $sql = "select dc_ch,dc_name,dc_network_id,dc_service_id from da_channel where dc_network_id = ? and dc_service_id = ?;";
    my $sth = $dbh->prepare($sql);
    my $rv = $sth->execute($networkid, $serviceid);
    if (my $rcols = $sth->fetch()) {
	my ($dc_ch, $dc_name, $dc_network_id, $dc_service_id) = @$rcols;
	$phych = $dc_ch;
    } else {
	print $q->header(-type=>$tsdownload::contenttype, -status=>406, -charset=>'utf-8', -connection=>'close', -cachecontrol=>'no-cache', -pragma=>'no-cache', -expires=>'-1');
	exit 1;
    }
    $dbh->disconnect();
}

binmode(STDOUT);
my $sdt = DateTime->from_epoch( epoch=>$stime, time_zone=>'local' );
my $edt = DateTime->from_epoch( epoch=>$etime, time_zone=>'local' );
my $request = sprintf("%02d%02d%02d-%02d%02d%02d-%02d%02d%02d-%d.ts", $sdt->year - 2000, $sdt->month, $sdt->day, $sdt->hour, $sdt->minute, $sdt->second, $edt->hour, $edt->minute, $edt->second, $phych);
my $s = RTMP->new($io, $tsdownload::tsdir);
my $content_length = 0;
$s->{file} = OneSeg24->new($s, $hdtv);
if (!$s->{file}->open($request)) {
    print $q->header(-type=>$tsdownload::contenttype, -status=>404, -charset=>'utf-8', -connection=>'close', -cachecontrol=>'no-cache', -pragma=>'no-cache', -expires=>'-1');
    exit 4;
}
for (my $i = 0; $i < @{$s->{file}->{files}}; $i++) {
    my $file = $s->{file}->{files}->[$i];
    $content_length += ($file->{end} - $file->{start});
}
my $ds = $title . '.ts';
if ($numb > 0) {
    $ds = $title . '#' . $numb . '.ts';
}
if ($q->user_agent() !~ m!(Firefox|Chrome|Safari)!) {
    $ds = uri_escape($ds);
}
$io->print("Content-Type: video/ts\nPragma: no-cache\nContent-Length: $content_length\nContent-Disposition: attachment; filename=\"$ds\"\n\n");
for (my $i = 0; $i < @{$s->{file}->{files}}; $i++) {
    my $file = $s->{file}->{files}->[$i];
    if (open(IN, "$file->{path}")) {
	seek(IN, $file->{offset}, 0);
	my $remain = $file->{end} - $file->{start};
	while ($remain > 0) {
	    my $rlen = $remain;
	    $rlen = $tsdownload::packsize if ($remain > $tsdownload::packsize);
	    my $recvlen = read(IN, my $buf, $rlen);
	    $io->write($buf, $recvlen);
	    $remain -= $rlen;
	}
	close(IN);
    }
}

1;
